package stack;

import static givenwhenthen.Given.given;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import givenwhenthen.Given;
import givenwhenthen.When;

import java.util.EmptyStackException;
import java.util.Stack;

import org.junit.Test;

public class StackSpecs {
    protected Stack<String> stack;

    Given aNewStack = new Given("a new stack") {
        @Override
        public void execute() {
            stack = new Stack<String>();
        }
    };

    When anItemAdded = new When("an item is pushed") {
        @Override
        public void execute() {
            stack.push("foo");
        }
    };

    private When pop = new When("an item is popped") {
        @Override
        public void execute() {
            stack.pop();
        }
    };

    @Test
    public void stackShouldNotBeEmptyAfterPush() {
        given(aNewStack).when(anItemAdded).then("the stack should not be empty");
        assertFalse(stack.isEmpty());
    }

    @Test
    public void stackShouldHaveASizeOf1AfterPush() {
        given(aNewStack).when(anItemAdded).then(
                "the stack should have a size of 1");
        assertEquals(1, stack.size());
    }

    @Test
    public void newStackShouldBeEmpty() {
        given(aNewStack).then("the stack should be empty");
        assertTrue(stack.isEmpty());
    }

    @Test
    public void newStackShouldHaveASizeOf0() {
        given(aNewStack).then("the stack should have a size of 0");
        assertEquals(0, stack.size());
    }

    @Test(expected = EmptyStackException.class)
    public void poppingFromNewStackShouldFail() {
        given(aNewStack).when(pop).then(
                "an EmptyStackException should be thrown");
    }
}
