/*
 See http://www.kerrybuckley.com/2007/01/09/given-when-then-in-junit
 Copyright (C) 2007 Kerry Buckley

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package givenwhenthen;

/**
 * Create an anonymous subclass of this class for each 'when' instance.
 * 
 * @author Kerry Buckley
 */
public abstract class When {
    private String desc;

    private RuntimeException thrown;

    public When(final String desc) {
        this.desc = desc;
    }

    /**
     * This method must be implemented to perform the behaviour corresponding to
     * the particular 'when' instance.
     */
    public abstract void execute();

    /**
     * Does not participate functionally in the test, but allows the 'when' and
     * 'then' descriptions to be reported. Also throws any exceptions deferred
     * in {@link Given#given(Given)}.
     * 
     * @param then
     *            a descriptive string
     */
    public void then(final String then) {
        report();
        reportThen(then);
        if (thrown != null) {
            throw thrown;
        }
    }

    public void setThrown(final RuntimeException t) {
        this.thrown = t;
    }

    static void reportThen(final String then) {
        System.out.println("Then:  " + then);
    }

    private void report() {
        System.out.println("When:  " + desc);
    }
}
